package com.hzya.frame.plugin.ht.service.impl;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.plugin.ht.entity.FundsAllocationEntity;
import com.hzya.frame.plugin.ht.service.IFundsAllocationPluginService;
import com.hzya.frame.plugin.ht.service.IFundsAllocationService;
import com.hzya.frame.seeyon.enums.ColEventTypeEnum;
import com.hzya.frame.seeyon.util.SM2Util;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import com.hzya.frame.web.exception.BaseSystemException;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description 资金拨付 流程结束回调 需要同时回调有度税务和中信司库
 * @Author xiangerlin
 * @Date 2025/6/21 14:17
 **/
@Service(value = "fundsAllocationPluginServiceImpl")
public class FundsAllocationPluginServiceImpl implements IFundsAllocationPluginService {
    Logger logger = LoggerFactory.getLogger(FundsAllocationPluginServiceImpl.class);
    @Value("${zt.url}")
    private String interfaceUrl;
    @Value("${zx.publicKey:}")
    private String publicKey;
    @Autowired
    private IFundsAllocationService fundsAllocationService;
    /**
     * 资金拨付流程结束回调有度税务系统和中信司库
     *
     * @param requestJson
     * @return
     * @throws BaseSystemException
     */
    @Override
    public JsonResultEntity callback(JSONObject requestJson) throws BaseSystemException {
        String dataSourceCode = requestJson.getString("apiDataSourceCode");//数据源
        String headerStr = requestJson.getString("headers");//表头
        String eventType = requestJson.getString("eventType");
        String summaryId = requestJson.getString("summaryId");
        Assert.notEmpty(dataSourceCode,"数据源不能为空");
        Assert.notEmpty(headerStr,"headers不能为空");
        Assert.notEmpty(eventType,"eventType不能为空");
        Assert.notEmpty(summaryId,"summaryId不能为空");
        JSONObject headers = requestJson.getJSONObject("headers");
        String formmainTableName = headers.getString("formmainTableName");
        JSONObject jsonStrObj = requestJson.getJSONObject("jsonStr");
        JSONObject businessData = jsonStrObj.getJSONObject("businessDataStr");
        JSONObject formmainData = businessData.getJSONObject(formmainTableName);
        if (null != formmainData){
            //税务系统id，这个字段不为空时表示是税务推过来的单子
            String field0061 = formmainData.getString("field0061");
            //司库系统批次号
            String field0063 = formmainData.getString("field0063");
            if (StrUtil.isNotEmpty(field0061) || StrUtil.isNotEmpty(field0063)){
                //查询审批意见
                FundsAllocationEntity fundAllocation = new FundsAllocationEntity();
                fundAllocation.setCol_summary_id(summaryId);
                List<FundsAllocationEntity> fundsAllocationList = fundsAllocationService.queryCtpCommentList(fundAllocation);
                if (CollectionUtils.isNotEmpty(fundsAllocationList)){
                    fundAllocation = fundsAllocationList.get(0);
                }
                String appId = "";//应用
                String apiCode = "";//api
                String param = "";//参数
                if (StrUtil.isNotEmpty(field0061)){
                    appId = "800060";
                    apiCode = "8000600008";
                    //税务
                    JSONObject jsonObject =  new JSONObject();
                    jsonObject.put("transferOrderNumber",field0061);
                    jsonObject.put("approvalUser",fundAllocation.getName());
                    jsonObject.put("approvalTime",fundAllocation.getApproval_time());
                    jsonObject.put("approvalRemark",StrUtil.isEmpty(fundAllocation.getContent()) ? "审批通过" : fundAllocation.getContent());
                    if (ColEventTypeEnum.ONPROCESSFINISHED.getType().equals(eventType)){
                        jsonObject.put("approvalStatus","2");
                    }else if (ColEventTypeEnum.ONSTOP.getType().equals(eventType)){//审批终止
                        jsonObject.put("approvalStatus","3");
                    }
                    param = jsonObject.toJSONString();
                }else if (StrUtil.isNotEmpty(field0063)){
                    appId = "800058";
                    apiCode = "8000580004";
                    //司库
                    JSONObject jsonObject =  new JSONObject();
                    jsonObject.put("batchNo",field0063);
                    jsonObject.put("reason",fundAllocation.getContent());
                    if (ColEventTypeEnum.ONPROCESSFINISHED.getType().equals(eventType)){
                        jsonObject.put("status","APPROVED");
                    }else if (ColEventTypeEnum.ONSTOP.getType().equals(eventType)){//审批终止
                        jsonObject.put("status","REJECTION");
                    }
                    JSONObject objectData = new JSONObject();
                    objectData.put("data",jsonObject);
                    logger.info("资金拨付审批回调司库请求参数明文:{}",objectData.toJSONString());
                    String encrypt = SM2Util.encrypt(objectData.toJSONString(),publicKey);
                    JSONObject jsonData = new JSONObject();
                    jsonData.put("data",encrypt);
                    param = jsonData.toJSONString();
                }
                if (StrUtil.isNotEmpty(appId) && StrUtil.isNotEmpty(apiCode) && StrUtil.isNotEmpty(param)){
                    logger.info("回调有度/司库请求apiCode{}，报文:{}",apiCode,param);
                    String result = HttpRequest.post(interfaceUrl)
                            .header("appId", appId)//税务/司库应用
                            .header("apiCode", apiCode)//回调接口
                            .header("publicKey", "ZJYA7f8FzV219otH8zhkReiyyWpXswpbY/+StvC2em0hf59Ce7eDIk+3zDUT+v578prj")//OA公钥
                            .header("secretKey", "xJ9J1Ev2F0faiJ/nQnCNklskAgtQp3QSm+ihO21uY/H0UADj0tSDPxmIhFfC4v6Fj3JzOP8MtA1LSGvL+2BWG8c/o7DKi92S4mr3zcGearA=")//OA密钥
                            .body(param)//表单内容
                            .timeout(30000)//超时，毫秒
                            .execute().body();
                    logger.info("回调有度/司库返回报文:{}",result);
                    return BaseResult.getSuccessMessageEntity(result);
                }else {
                    logger.info("appId、apiCode、param 其中有参数是空的，不能执行");
                }
            }else {
                logger.info("不是有度税务/中信司库推过来的单子，不需要执行");
            }
        }
        return BaseResult.getFailureMessageEntity("表单数据为空");
    }
}
